"""This module contains Requirements class, which is the core of the current process."""
import os
import re
from enum import Enum
from typing import List, Tuple


class RequirementsFormat(str, Enum):
    pip_list = "pip_list"
    pip_freeze = "pip_freeze"


class Requirements:
    """
    A class that handles parsing and rendering requirements data.

    Attributes:
        None

    Methods:
        _read_file_contents: Read the contents of a file.
        _from_pip_freeze: Parse data in the pip freeze format.
        _from_pip_list: Parse data in the pip list format.
        render: Render the requirements data based on the specified format.
    """

    def _merge_multiple_requirements(self, source_path: str) -> set:
        requirements = set()
        paths = os.listdir(source_path)
        paths = [
            os.path.join(source_path, path) for path in paths if path.endswith(".txt")
        ]

        for file in paths:
            with open(file, "r") as src:
                requirements.update(src.read().splitlines())

        return requirements

    def _read_file_contents(self, source_path: str) -> str:
        """
        Read the contents of a file.

        Args:
            source_path: The path to the file to be read.

        Returns:
            The contents of the file as a string.
        """
        basepath = os.path.abspath(source_path)
        if os.path.isdir(basepath):
            return self._merge_multiple_requirements(basepath)
        with open(basepath, "r") as src:
            return src.read().splitlines()

    def _from_pip_freeze(self, data: str) -> List[Tuple[str, str]]:
        """
        Parse data in the pip freeze format.

        Args:
            data: The pip freeze formatted data to be parsed.

        Returns:
            A list of tuples containing package names and versions.
        """

        lines = [line for line in data if not line.startswith("#")]
        pattern = r"(==|<=|>=|<|>)"
        package_data = [re.split(pattern, line) for line in lines]
        return [
            (package[0], package[2]) if len(package) > 1 else package
            for package in package_data
        ]

    def _from_pip_list(self, data: str) -> List[Tuple[str, str]]:
        """
        Parse data in the pip list format.

        Args:
            data: The pip list formatted data to be parsed.

        Returns:
            A list of tuples containing package names and versions.
        """

        package_data = [tuple(line.split()) for line in data[2:]]
        return [(package[0], package[1]) for package in package_data]  # pylint

    def render(self, source_path: str, format: str) -> List[Tuple[str, str]]:
        """
        Render the requirements data based on the specified format.

        Args:
            source_path: The path to the requirements file.
            format: The format of the requirements file (e.g., "pip_freeze", "pip_list"). # noqa

        Returns:
            A list of tuples containing package names and versions.
        """
        data = self._read_file_contents(source_path)
        if format == "pip_freeze":
            return self._from_pip_freeze(data)
        if format == "pip_list":
            return self._from_pip_list(data)



import re
from typing import Dict

import requests

from extractor.logger import logger


class StandardCheck:
    def __init__(self):
        self.project_default_error = "No project url found, please check manually"
        self.version_default_error = "No version url found, please check manually"

    def gh_pattern(self, pattern: str, url: str, custom_error: str = None):
        try:
            found = re.search(pattern, url)
        except Exception as e:
            logger.warning(custom_error) if custom_error else logger.warning(e)
        else:
            return found

    def additional_urls(self, pattern: str, project_urls: str):
        urls = project_urls.items()
        possible_keys = [
            "Code",
            "Source Code",
            "Source code",
            "Source",
            "Homepage",
            "Home",
            "Repository",
            "repository",
            "Download",
            "download",
        ]
        for title, url in urls:
            if _ := self.gh_pattern(pattern, url) and title in possible_keys:
                return url

    def licenses(self, raw_data: Dict) -> str:
        licenses = raw_data["license"]
        if licenses != "" and licenses is not None:
            return licenses
        licenses_pattern = r"^[lL]icense.*"

        for lic in raw_data["classifiers"]:
            if license_found := re.search(licenses_pattern, lic):
                return license_found.group()

    def project_url(self, pattern: str, project_url: str, project_urls: Dict) -> str:
        if project_url != "" and self.gh_pattern(pattern, project_url):
            return project_url

        if project_urls:
            logger.debug("Nested metadata found")
            return self.additional_urls(pattern, project_urls)

    def _url_exists(self, url) -> bool:
        response = requests.get(url)
        return response.status_code != 404

    def _version_handler(self, filtered_data: Dict, version: str):
        exceptions = {
            "pandas": f"v{version}",
            "azure-keyvault-secrets": f"azure-keyvault-secrets_{version}",
            "azure-storage-blob": f"azure-storage-blob_{version}",
            "pyspark": f"pyspark_v{version}",
            "loguru": f"{filtered_data['project_url'].rsplit('/', 1)[0].replace('archive','')}tree/{version}/",
        }

        version = exceptions.get(filtered_data["name"], version)

        if version and version.startswith("azure"):
            return f"{filtered_data['project_url'].replace('main',version)}"

        if version and version.startswith("pyspark"):
            return f"{filtered_data['project_url'].replace('master',version).replace('pyspark_','')}"

        if "loguru" in version:
            return version

        default_url = f"{filtered_data['project_url']}/tree/{version}/"

        return (
            default_url
            if self._url_exists(default_url)
            else f"{filtered_data['project_url']}/tree/v{version}/"
        )

    def version(self, version: str, pattern: str, filtered_data: Dict) -> Dict:
        if version and self.gh_pattern(
            pattern, filtered_data.get("project_url"), self.project_default_error
        ):
            project_url = self._version_handler(filtered_data, version)
            if _ := self._url_exists(project_url):
                filtered_data["version_url"] = self._version_handler(
                    filtered_data, version
                )
            else:
                filtered_data["version_url"] = self.version_default_error
        else:
            filtered_data["version_url"] = self.version_default_error
        return filtered_data



import os
from pathlib import Path
from typing import Dict

import pandas as pd
import requests
from ghapi.all import GhApi
from rich.progress import track

from extractor.checks import StandardCheck
from extractor.logger import logger
from extractor.render import Requirements

PYPI_URL_BASE = "https://pypi.org/pypi"
GITHUB_URL_BASE = "https://github.com"
IAC_EXCEPTIONS_LIST = [
    "hashicorp/terraform",
    "databricks/terraform-provider-databricks",
]


def is_python(project: str) -> bool:
    """
    Check if the project is a Python project.

    Args:
        project: The name of the project.

    Returns:
        A boolean value.
    """
    return project.startswith("python/")


def is_iac_project(project: str) -> bool:
    """
    Check if the project is an IAC project.

    Args:
        project: The name of the project.

    Returns:
        A boolean value.
    """
    return project in IAC_EXCEPTIONS_LIST


def get_raw_data_from_github(project: str, version: str) -> Dict[str, str]:
    """
    Retrieve raw metadata for a project from a given URL.

    Args:
        project: The name of the project.
        version: The version of the project.

    Returns:
        A dictionary containing the raw metadata of the project.
    """
    api = GhApi()
    owner, repo = project.split("/")
    data = api.repos.get_content(owner=owner, repo=repo, path="LICENSE", ref=version)
    license_url = data.get("html_url")
    return {
        "name": project,
        "version": version,
        "license": license_url,
        "pypi_release_url": "",
        "version_url": license_url.replace("/LICENSE", ""),
    }


def get_raw_data_from_pypi(project: str) -> Dict[str, str]:
    """
    Retrieve raw metadata for a project from a given URL.

    Args:
        project: The name of the project.

    Returns:
        A dictionary containing the raw metadata of the project.
    """
    try:
        r = requests.get(
            f"{PYPI_URL_BASE}/{project}/json",
            headers={"Accept": "application/json"},
            timeout=5,
        )
    except Exception as e:
        logger.error(e)
    else:
        if r.status_code == 404:
            logger.error(f"Project {project} not found")
            return {}
        return r.json()["info"]


def get_raw_data(project: str, version: str = "") -> Dict[str, str]:
    """
    Retrieve raw metadata for a project from a given URL.

    Args:
        project: The name of the project.

    Returns:
        A dictionary containing the raw metadata of the project.
    """
    if is_iac_project(project) or is_python(project):
        return get_raw_data_from_github(project, version)
    else:
        return get_raw_data_from_pypi(project)


def filter_data(raw_data: Dict[str, str], version: str) -> Dict[str, str]:
    """
    Filter relevant metadata from raw data.

    Args:
        raw_data: The raw metadata of a project.
        version: The version of the project.

    Returns:
        A dictionary containing filtered metadata.
    """
    if raw_data:
        project_name = raw_data["name"]
        project_url = raw_data["project_url"]
        project_urls = raw_data["project_urls"]
        project_version = version or raw_data["version"]
        check = StandardCheck()
        pypi_url = f"https://pypi.org/project/{project_name}/{project_version}/"
        gh_url_pattern = r"(https:\/\/|http:\/\/)github\.com"
        project_url = check.project_url(gh_url_pattern, project_url, project_urls)
        filtered_data = {
            "name": project_name,
            "version": project_version,
            "license": check.licenses(raw_data),
            # "homepage": raw_data["home_page"],
            "pypi_release_url": pypi_url,
            "project_url": check.project_url(gh_url_pattern, project_url, project_urls),
        }

        logger.info(f"Searching GitHub url for: {project_name}")

        filtered_data = check.version(version, gh_url_pattern, filtered_data)
        filtered_data["version_url"] = (
            filtered_data["version_url"]
            if filtered_data["version_url"]
            not in [check.project_default_error, check.version_default_error]
            else pypi_url
        )
        del filtered_data["project_url"]
        return filtered_data

    return {}


def extract_data(source_path: Path, format: str) -> None:
    """
    Extract data based on the specified requirements format.

    Args:
        source_path: The path to the requirements file.
        format: The format of the requirements file.

    Returns:
        pd.DataFrame
    """
    logger.info("Starting process")
    logger.debug(f"Retrieving: {source_path}")
    result = Requirements().render(source_path, format)
    pkgs_raw_metadata = []
    for pkg in track(result):
        if is_iac_project(pkg[0]) or is_python(pkg[0]):
            filtered_data = get_raw_data(pkg[0], pkg[1])
        else:
            filtered_data = filter_data(
                get_raw_data(pkg[0]), pkg[1] if len(pkg) > 1 else None
            )
        if filtered_data:
            pkgs_raw_metadata.append(filtered_data)
    output = pd.DataFrame(pkgs_raw_metadata)
    # output["uppercased_name"] = output["name"].str.upper()
    # output = output.sort_values(by=["uppercased_name"])
    # output = output.drop_duplicates(subset=["uppercased_name"], keep="first")
    # del output["uppercased_name"]
    output["uppercased_version_url"] = output["version_url"].str.upper()
    output = output.sort_values(by=["uppercased_version_url"])
    output = output.drop_duplicates(subset=["uppercased_version_url"], keep="first")
    del output["uppercased_version_url"]
    return output


def save_data(data: pd.DataFrame, output: Path):
    # Extract directory from the output string
    raw_name = str(output).split(".")[0]
    output_directory = os.path.dirname(output)
    only_version_url_list = data.copy()
    only_version_url_list["Commit ID"] = only_version_url_list["version_url"]
    # only_version_url_list["Version Number"] = only_version_url_list["version"]
    only_version_url_list_df = only_version_url_list[["Commit ID"]]
    # only_version_url_list_df = only_version_url_list_df.copy().pivot_table(by=["Commit ID", "Version Number"])
    logger.info(f"Storing into: {output}")
    if not os.path.exists(output_directory):
        os.makedirs(output_directory)
    if str(output).endswith(".csv"):
        data.to_csv(output, index=False)
        only_version_url_path = f"{raw_name}_only_version_urls.csv"
        only_version_url_list_df.to_csv(only_version_url_path, index=False)
        logger.info(f"Only urls version stored into: {only_version_url_path}")
        logger.info("All done! Have a Great day")
    elif str(output).endswith(".xlsx"):
        data.to_excel(output, index=False)
        only_version_url_path = f"{raw_name}_only_version_urls.xlsx"
        only_version_url_list_df.to_excel(only_version_url_path, index=False)
        logger.info(f"Only urls version stored into: {only_version_url_path}")
        logger.info("All done! Have a Great day")
    else:
        logger.error("Not supported format.")



from importlib.metadata import version as pkg_version
from pathlib import Path

import typer
from rich import print
from typing_extensions import Annotated

from extractor.core import extract_data, save_data
from extractor.render import RequirementsFormat

app = typer.Typer()
VERSION = pkg_version("pymetasnap")


@app.command(name="version")
def version():
    return print(VERSION)


@app.command(name="extract")
def main(
    source_path: Annotated[
        Path,
        typer.Option(
            exists=True,
            file_okay=True,
            dir_okay=True,
            writable=True,
            readable=True,
            resolve_path=True,
            prompt=True,
            help="Requirements files paths. Can be a single file, or a folder",
        ),
    ] = "",
    output: Annotated[
        Path,
        typer.Option(
            file_okay=True,
            dir_okay=False,
            readable=True,
            resolve_path=True,
            prompt=True,
            help="Path to store the data",
        ),
    ] = "",
    format: Annotated[
        RequirementsFormat,
        typer.Option(prompt=True, help="Incoming requirements format."),
    ] = RequirementsFormat.pip_freeze,
):  # pragma: no cover
    data = extract_data(source_path, format)
    save_data(data, output)


if __name__ == "__main__":
    app()



from loguru import logger
from rich.logging import RichHandler

# <green>{time:YYYY-MM-DD HH:mm:ss}</green> | <level>{level}</level> |

logger.remove()
logger.add(RichHandler(), level="INFO", format="<level>{message}</level>")
